#include <basic.h>
#include <bstdio.h>
#include <bstdlib.h>
#include <tstring.h>
#include <tcode.h>
#include <tctype.h>
#include <btron/dp.h>
#include <btron/clk.h>
#include <tad.h>
#include <new>
#include "ktext.h"
#include "gvar.h"
#include "gprog.h"
#include "subc.h"
#include "message.h"
#include "r_obj.h"

extern OutSider grovalFunc;
extern W wid0;

#define LEX_STRING_BUFF_SIZE	100

GProgramText::GProgramText()
{
  string = new(std::nothrow) TC[LEX_STRING_BUFF_SIZE];
  src_code = NULL;
  v_segData = NULL;
}

GProgramText::GProgramText(RealObject *src)
{
  string = new(std::nothrow) TC[100];
  this->setText(src);
  v_segData = NULL;
}

GProgramText::~GProgramText()
{
  delete[] string;
  if (v_segData != NULL){
    delete[] v_segData;
  }
}

void GProgramText::setText(RealObject *src)
{
  src_code = src;
  seg = src_code->get_segment();
}

TC GProgramText::getc2()
{
  TC c;

  if (seg == NULL){
    return EOF;
  }
  c = seg->getw();
  if (c == (TC)EOF){
    seg = src_code->get_segment();
    if (seg == NULL){
      return EOF;
    }
    c = seg->getw();
  }
  return c;
}

TC GProgramText::getc()
{
  if (seg == NULL){
    return EOF;
  }
  TC c;
  while (seg->getKind() != 0){
// b_printf("fusen segment \n");
    if (seg->getKind() == 0xe2){
      return EOF;
    }
    if ((seg = src_code->get_segment()) == NULL){
      return EOF;
    }
  }
//b_printf("data segment \n");
  if ((c = seg->getw()) == (TC)EOF){
    seg = src_code->get_segment();
    if ((seg == NULL) || (seg->getKind() == 0xe2)){
      return EOF;
    }
    while (seg->getKind() != 0){
      seg = src_code->get_segment();
      if ((seg == NULL) || (seg->getKind() == 0xe2)){
        return EOF;
      }
    }
    c = seg->getw();
  }
// b_printf("c = %lc \n", c);
  return c;
}

TC GProgramText::ungetc(TC c)
{
  seg->ungetw(c);
  return c;
}

void GProgramText::skip_for(TC c)
{
  while (getc() != c);
}

SYMBOL GProgramText::lex()
{
  TC c, *p;

  for (;;){
    c = getc2();
    if (seg->getKind() != 0){
      if (seg->getKind() == TS_VOBJ){
        v_link = seg->get_vobj();
        if (v_segData != NULL){
          delete [] v_segData;
          v_segData = NULL;
        }
        v_segData = seg->get_seg(v_segData);
        src_code->get_segment();
        return (rtn = V_OBJ);
      }

      if (seg->getKind() == TS_TEXTEND){
        return (rtn = EOFSYM);
      }
      
      if (src_code->get_segment() == NULL){
        return (rtn = EOFSYM);
      }
    }
    else {
      if ((c != TK_KSP) && (c != TK_TAB) && (c != TK_NL)){
        break;
      }
    }
  }
	
  if (c == (TC)EOF){
    return (rtn = EOFSYM);
  }
	
  if ((c == L'') || (tc_isdigit(c) != 0)){
    ungetc(c);
    number = geti();
    return (rtn = NUMBER);
  }
  if(c == TK_SQOT){
    number = getc() & 0x00ff;
    if (getc() != TK_SQOT){
      return (rtn = ERRSYM);
    }
    return (rtn = NUMBER);
  }
	
  if ((tc_isalpha(c) != 0) || (c == L'') || (c == L'')){
    p = string;
    *p++ = c;
    while (((c = getc()) == (TC)EOF)
           || (tc_isalnum(c) != 0) || (c == L'')){
      *p++ = c;
    }
    ungetc(c);
    *p = TNULL;
    return (rtn = VAR_STRING);
  }
  int cnt = 1;
  if (c == L''){
    p = string;
    for(;;){
      if ((c = getc()) == TK_DQOT){
        if (cnt > 1){
          p--;
          if (*p != TK_BSLH){
            p++;
            break;
          }
        }
        else {
          break;
        }
      }
      if (++cnt > LEX_STRING_BUFF_SIZE){
        break;
      }
      *p++ = c;
    }
    *p = TNULL;
    return (rtn = STRING);
  }
	
  switch (c){
  case L'':
    if ((c = getc()) == L''){
      return (rtn = EQU);
    }
    else {
      ungetc(c);
      return (rtn = BECOMES);
    }
  case L'':
    if ((c = getc()) == L''){
      return (rtn = GEQ);
    }
    else {
      ungetc(c);
      return (rtn = GTR);
    }
  case L'':
    if ((c = getc()) == L''){
      return (rtn = LEQ);
    }
    else {
      ungetc(c);
      return (rtn = LSS);
    }
  case L'':
    if ((c = getc()) != L''){
      ungetc(c);
//      cout << "! error" << endl;
    }
    return (rtn = NEQ);
  case L'':
    if ((c = getc()) != L''){
      ungetc(c);
//	cout << "| error" << endl;
    }
    return (rtn = OR);
  case L'':
    if ((c = getc()) != L''){
      ungetc(c);
//	cout << "& error" << endl;
    }
    return (rtn = AND);
  case L'':
    return (rtn = SEMICOL);
  case L'':
    return (rtn = COMMA);
  case L'':
    return (rtn = PLUS);
  case L'':
    return (rtn = MINUS);
  case L'':
    return (rtn = TIMES);
  case L'':
    c = getc();
    if(c != L''){
      ungetc(c);
      return (rtn = SLASH);
    }
    else {
      skip_for(TK_NL);
      return(lex());
    }
  case L'':
    return (rtn = MOD);
  case L'':
    return (rtn = LPAR);
  case L'':
    return (rtn = RPAR);
  case L'':
    return (rtn = LBRACE);
  case L'':
    return (rtn = RBRACE);
  case L'':
    return (rtn = LBRCKT);
  case L'':
    return (rtn = RBRCKT);
  default:
    string[0] = c;
    string[1] = TNULL;
    return (rtn = ERRSYM);
  }
}


W GProgramText::geti()
{
  TC str[100], *p;
  p = &str[0];
	
  TC c = getc();
  if (tc_isdigit(c) != 0){
    *p++ = c;
    p = getDigit(p);
    c = getc();
  }
  if (c == L''){
//		*p++ = c;
    p = getDigit(p);
    c = getc();
  }
  if (c == L'' || c == L''){
//		*p++ = c;
    c = getc();
    if (c == L'' || c == L''){
//			*p++ = c;
    }
    else{
      ungetc(c);
    }
    p = getDigit(p);
    c = getc();
  }
  ungetc(c);
  *p = TNULL;
  return tc_atoi(str);
}

TC *GProgramText::getDigit(TC *p)
{
  TC c;
  
  c = getc();
  // while (tc_isdigit(c = getc()) != 0){ -- tctype.hǤϥޥʤΤǤޤ
  while (tc_isdigit(c) != 0){
    *p++ = c;
    c = getc();
  }
  ungetc(c);
	
  return p;
}

///////////////////////////////////////////////////////////////////////////

GProgram::GProgram()
{
  progText = new(std::nothrow) GProgramText;
  varList = new(std::nothrow) GVariableList;
  progList = new(std::nothrow) GProgramList;
  mes = new(std::nothrow) CMessage;
}

GProgram::~GProgram()
{
  delete progText;
  delete varList;
  delete progList;
  delete mes;
}

int GProgram::variable()
{
  int flg = 0;
  if (progText->lex() == LBRCKT){
    progText->lex();
    expression();
    if (progText->getReturn() != RBRCKT){
      mes->error_mes(NULL, SYNTAX);
// b_printf("[statement] var error NOT RBRCKT \n");
      return flg;
    }
    progText->lex();
    flg++;
  }
  return flg;
}

void GProgram::condition()
{
  expression();
	
  SYMBOL c = progText->getReturn();
  if ((c == EQU) || (c == NEQ) || (c == LSS) || 
      (c == GTR) || (c == LEQ) || (c == GEQ)){
    progText->lex();
    expression();
  }
  else {
    mes->error_mes(NULL, SYNTAX);
// b_printf("[condition] operator error \n");
    return;
  }
  progList->add_op(c);
}

void GProgram::factor()
{
  TC str[100];
  W d;
  int flg;
  SYMBOL c = progText->getReturn();
	
  if (c == VAR_STRING){
    GVariable *p;
    tc_strcpy(str, progText->getString());
    if ((p = varList->getObj(str)) == NULL){;
      mes->error_mes(str, VAR_NODEFINE);
// b_printf("[factor] no define variable : \n");
      return;
    }
    if (p->IsKindOf() == VAR){
      flg = variable();
			
      if (flg == 0){
        progList->add_var(0, p);
      }
      else {
        progList->add_var(1, p);	// array valiable
      }
    }
    else if (p->IsKindOf() == FUNC){
      function();
    }
    else {
      mes->error_mes(str, NODEFINE);
// b_printf("[factor] no defined string< %ls > \n", str);
      return;
    }
  }
  else if (c == NUMBER){
    d = progText->getNumber();
    progList->add_const(d);
    progText->lex();
  }
  else if (c == LPAR){
    progText->lex();
    expression();
    if (progText->getReturn() != RPAR){
      mes->error_mes(NULL, SYNTAX);
// b_printf("[factor] error () \n");
      return;
    }
    progText->lex();
  }
  else {
    mes->error_mes(NULL, SYNTAX);
// b_printf("factor err \n");
    return;
  }
}

void GProgram::term()
{
  SYMBOL c;
	
  factor();
  c = progText->getReturn();
  while ((c == TIMES) || (c == SLASH) || (c == MOD)){
    progText->lex();
    factor();
    progList->add_op(c);
    c = progText->getReturn();
  }
}

void GProgram::expression()
{
  SYMBOL c = progText->getReturn();
  if ((c == PLUS) || (c == MINUS) 
      || (c == GTR) || (c == LSS)
      || (c == GEQ) || (c == LEQ)
      || (c == EQU) || (c == NEQ)
      || (c == OR) || (c == AND)){
    progText->lex();
  }
  term();
  if (c == MINUS){
    progList->add_um();
  }
  c = progText->getReturn();
  while ((c == PLUS) || (c == MINUS) 
         || (c == GTR) || (c == LSS)
         || (c == GEQ) || (c == LEQ)
         || (c == EQU) || (c == NEQ)
         || (c == OR) || (c == AND)){
    progText->lex();
    term();
    progList->add_op(c);
    c = progText->getReturn();
  }
}

void GProgram::assign()
{
  TC str[100];
  GVariable *p;
  int flg;
	
  tc_strcpy(str, progText->getString());
  if ((p = varList->getObj(str)) == NULL){;
    mes->error_mes(str, VAR_NODEFINE);
// b_printf("[factor] no define variable : %ls \n", str);
    return;
  }
  if (p->IsKindOf() != VAR){
    mes->error_mes(str, NODEFINE);
// b_printf("[factor] This is not variable name : %ls \n", str);
    return;
  }
	
  flg = variable();
	
  if (progText->getReturn() != BECOMES){ // =
    mes->error_mes(NULL, SYNTAX);
// b_printf("error statement '=' \n");
  }
  progText->lex();
  expression();
	
  if (flg == 0){
    progList->add_op2((TC[]){L""}, p); // add code
  }
  else {
    progList->add_op2((TC[]){L"Ρϡ"}, p); // add code
  }
}

void GProgram::argument()
{
  GVariable *p;
  int cnt = 0;
  SYMBOL c;
	
  if ((p = varList->getObj((TC[]){L""})) == NULL){;
    mes->error_mes((TC[]){L""}, VAR_NODEFINE);
// b_printf("[argument] no define variable : $[] \n");
    return;
  }
	
  c = progText->getReturn();
  while (c != RPAR){
    if (cnt > 9){
      mes->error_mes(NULL, SYNTAX);
// b_printf("[argument] too many arguments. \n");
      return;
    }
		
    progList->add_const(cnt); // add code
    cnt++;
    expression();
    progList->add_op2((TC[]){L"Ρϡ"}, p); // add code
		
    c = progText->getReturn();
    if ((c != COMMA) && (c != RPAR)){
      mes->error_mes(NULL, SYNTAX);
// b_printf("[argument] error , \n");
      return;
    }
    if (c == COMMA){
      c = progText->lex();
    }
  }
}

void GProgram::function()
{
  TC str[100];
  GVariable *p;
  GCalcFunc *CallFuncP;
	
  tc_strcpy(str, progText->getString());
  if ((p = varList->getObj(str)) == NULL){;
    mes->error_mes(str, FUNC_NODEFINE);
// b_printf("[function] not define function : %ls \n", str);
    return;
  }
  if (p->IsKindOf() != FUNC){
    mes->error_mes(str, VAR_DEFINE);
// b_printf("[function] This string < %ls > is not a function. \n", str);
    return;
  }
	
  if (progText->lex() == LPAR){
    progText->lex();
    argument();
    if (progText->getReturn() != RPAR){
      mes->error_mes(NULL, SYNTAX);
// b_printf("[function] error ) \n");
      return;
    }
  }
	
  CallFuncP = progList->add_func(); // add code
  CallFuncP->setNext2(p->getp());
	
  progText->lex();
  return;
}

void GProgram::if_proc()
{
  TC str[100];
  GCalcIf *IfStartP;
  GCalc *trueProgramEnd;
  GCalcDummy *NextProgramStart;
  SYMBOL c = progText->getReturn();
	
  if (c != LPAR){
    mes->error_mes(NULL, SYNTAX);
// b_printf("[if_proc] : error statement '(' \n");
    return;
  }
  progText->lex();
//	condition();
  expression();
  c = progText->getReturn();
  if(c != RPAR){
    mes->error_mes(NULL, SYNTAX);
// b_printf("[if_proc] : error statement ')' \n");
    return;
  }
  IfStartP = progList->add_if(); // add code
	
  progText->lex();
  statement();
  trueProgramEnd = getProgramEnd();
  if (progText->getReturn() == VAR_STRING){
    tc_strcpy(str, progText->getString());
    if(tc_strcmp(str, (TC[]){L""}) == 0){
      progText->lex();
      statement();
    }
  }
  NextProgramStart = progList->add_dummy(); // add code
  trueProgramEnd->setNext2(NextProgramStart);
  IfStartP->setFalseP(trueProgramEnd->getNext());
}

void GProgram::while_proc()
{
  SYMBOL c;
  GCalcWhile *WhileStartP;
  GCalc *trueProgramEnd;
  GCalcDummy *NextProgramStart;
  GCalcDummy *WhileProgramStart = progList->add_dummy(); // add code
	
  c = progText->getReturn();
  if (c != LPAR){
    mes->error_mes(NULL, SYNTAX);
// b_printf("[while_proc] : error statement '(' \n");
    return;
  }
  progText->lex();
//	condition();
  expression();
  c = progText->getReturn();
  if (c != RPAR){
    mes->error_mes(NULL, SYNTAX);
// b_printf("[while_proc] : error statement ')' \n");
    return;
  }
  WhileStartP = progList->add_while(); // add code
	
  progText->lex();
  statement();
  trueProgramEnd = getProgramEnd();
	
  NextProgramStart = progList->add_dummy(); // add code
	
  trueProgramEnd->setNext2(WhileProgramStart);
  WhileStartP->setFalseP(NextProgramStart);
	
// b_printf("end while_proc() \n");
}

void GProgram::vopen_proc()
{
  SYMBOL c = progText->getReturn();
	
  if (c == LPAR){
    if (progText->lex() != V_OBJ){
      mes->error_mes(NULL, SYNTAX);
// b_printf("[vopen_proc] error NOT V_OBJ \n");
      return;
    }
// b_printf("vopen function \n");
    progList->add_vopen(progText->getLink(), progText->getSegment());
    if (progText->lex() != RPAR){
      mes->error_mes(NULL, SYNTAX);
// b_printf("[vopen_proc] error NOT ')' \n");
      return;
    }
    if (progText->lex() != SEMICOL){
      mes->error_mes(NULL, SYNTAX);
// b_printf("[vopen_proc] error NOT ';' \n");
      return;
    }
    progText->lex();
  }
  else {
    mes->error_mes(NULL, SYNTAX);
// b_printf("[vopen_proc] error NOT '(' \n");
  }
}

void GProgram::println_proc()
{
  TC *p;
  SYMBOL c = progText->getReturn();
	
  if (c == LPAR){
    c = progText->lex();
    while (1){
      switch (c){
      case EOFSYM:
        mes->error_mes(NULL, SYNTAX);
// b_printf("[function] error EOFSYM \n");
        return;
      case RPAR:
	if (progText->lex() != SEMICOL){
          mes->error_mes(NULL, SYNTAX);
// b_printf("[println_proc] error NOT SEMICOL \n");
          return;
        }
        progText->lex();
        return;
      case STRING:
	p = progText->getString();
        progList->add_println(p);
        progText->lex();
        break;
      default:
	expression();
        progList->add_println((TC*)NULL);
        break;
      }
      c = progText->getReturn();
      if (c == COMMA){
        c = progText->lex();
      }
      else if (c != RPAR){
        mes->error_mes(NULL, SYNTAX);
// b_printf("[println_proc] error NOT ',' and NOT ')' \n");
        return;
      }
    }
  }
  else {
    mes->error_mes(NULL, SYNTAX);
// b_printf("[println_proc] error NOT '(' \n");
  }
}

void GProgram::statement()
{
  TC str[LEX_STRING_BUFF_SIZE];
  GVariable *calcObj;
  SYMBOL c;
  VARKIND flg;
	
  c = progText->getReturn();
  if (c == VAR_STRING){
    tc_strcpy(str, progText->getString());
    if (tc_strcmp(str, (TC[]){L""}) == 0){
      progText->lex();
      if_proc();
    }
    else if (tc_strcmp(str, (TC[]){L""}) == 0){
      progText->lex();
      while_proc();
    }
    else if (tc_strcmp(str, (TC[]){L""}) == 0){
      if (progText->lex() != SEMICOL){
        expression();
      }
      else {
        progList->add_push();
      }
      if (progText->getReturn() != SEMICOL){
        mes->error_mes(NULL, SYNTAX);
// b_printf("[statement] error statement ';' \n");
        return;
      }
      progList->add_return(); // add code
      progText->lex();
    }
    else if (tc_strcmp(str, (TC[]){L""}) == 0){
      progText->lex();
      println_proc();
    }
    else if (tc_strcmp(str, (TC[]){L""}) == 0){
      progText->lex();
      vopen_proc();
    }
    else {
      calcObj = varList->getObj(str);
      if (calcObj != NULL){
        flg = calcObj->IsKindOf();
        if (flg == VAR){
          assign();             // variable = expression;
        }
        else if (flg == FUNC){
          function();           // function();
          progList->add_pop();
        }
        else {
          return;
        }
        if (progText->getReturn() != SEMICOL){
          mes->error_mes(NULL, SYNTAX);
// b_printf("[statement] error statement ';' \n");
          return;
        }
        progText->lex();
      }
      else {
        mes->error_mes(str, FUNC_NODEFINE);
// b_printf("[function] not define function : %ls \n", str);
        return;
      }
    }
  }
  else if (c == LBRACE){
    c = progText->lex();
    while (c != RBRACE){
      statement();
      c = progText->getReturn();
    }
    progText->lex();
  }
  else if (c == SEMICOL){
    progText->lex();
  }
  else{
    mes->error_mes(NULL, SYNTAX);
// b_printf("[statement] : paser error \n");
  }
}

static struct {
  TC *name;
  W val;
} constVariable[] = {
//	"PI",		 3.14159265358979323846,
//	"E",		 2.71828182845904523536,
//	"GAMMA",	 0.57721566490153286060,
//	"DEG",		57.29577951308232087680,
//	"PHI",		 1.61803398874989484820,
  {NULL, 0}
};

//	If you add the function in this structure, 
//	add the function in the GCalcConstFunction::run().

static TC literal_abs[] = {L""};
static TC literal_rand[] = {L""};
static TC literal_locate[] = {L""};
static TC literal_cls[] = {L""};
static TC literal_lcls[] = {L"塲"};
static TC literal_getcurx[] = {L""};
static TC literal_getcury[] = {L""};
static TC literal_getkey[] =  {L""};
static TC literal_getkey2[] = {L""};
static TC literal_keyfl[] = {L""};
static TC literal_beep[] = {L""};
static TC literal_rsopen[] = {L"󡲣"};
static TC literal_rsclose[] = {L"󡲣"};
static TC literal_rsinit[] = {L"󡲣"};
static TC literal_rsputc[] = {L"󡲣"};
static TC literal_rsckb[] = {L"󡲣"};
static TC literal_rsgetc[] = {L"󡲣"};
static TC literal_qvgetp[] = {L""};
static TC literal_sleep[] = {L""};

static struct{
  TC *name;
  int argc;
} constFunction[] = {
  {literal_abs, 1},
  {literal_rand, 1},
  {literal_locate, 2},
  {literal_cls, 0},
  {literal_lcls, 1},
  {literal_getcurx, 0},
  {literal_getcury, 0},
  {literal_getkey, 0},
  {literal_getkey2, 0},
  {literal_keyfl, 0},
  {literal_beep, 2},
  {literal_rsopen, 1},
  {literal_rsclose, 1},
  {literal_rsinit, 5},
  {literal_rsputc, 2},
  {literal_rsckb, 1},
  {literal_rsgetc, 1},
  {literal_qvgetp, 2},
  {literal_sleep, 1},
  {TNULL, 0},
};

void GProgram::init()
{
  GVariable *pDollar;
  GCalcDummy *FunctionStart;
  GVariable *funcVar;
/*	
	while(constVariable[n].name != NULL){
		varList->add(constVariable[n].name, 1);
		GVariable *p =varList->getObj(constVariable[n].name);
		p->set(constVariable[n].val);
		n++;
	}
*/	
  varList->add((TC[]){L""}, 10); // var $[10];
  pDollar = varList->getObj((TC[]){L""});
	
  int n = 0;
  while (constFunction[n].name != NULL){
    varList->add(constFunction[n].name, FUNC);
    FunctionStart = progList->add_dummy();	// add code
    funcVar = varList->getObj(constFunction[n].name);
    funcVar->setp(FunctionStart); // set function start address
    for (int i = 0; i < constFunction[n].argc; i++){
      progList->add_const(i);
      progList->add_var(1, pDollar); // push $[i]
    }
    progList->add_constFunc(constFunction[n].name); // add code
    progList->add_return(); // add code
    n++;
  }
}

void GProgram::compile_init()
{
  progList->deleteAll();
  varList->deleteAll();
  this->init();
  grovalFunc.cls();
  mes->opening();
}

void GProgram::compile_end()
{
  mes->ending();
}

void GProgram::compile(RealObject *p, SRC_TYPE src_type)
{
  GProgramText *pt_buf = progText;
  GProgramText *pt_new = new(std::nothrow) GProgramText;
  progText = pt_new;
  progText->setText(p);
	
  SYMBOL c;
  TC str[100];
  TC func_name[100];
  W num = 1;
  GVariable *funcVar;
  GCalcDummy *FunctionStart;
  GVariable *p1, *p2;

  progText->lex();
  while ((c = progText->getReturn()) != EOFSYM){
    if (c == V_OBJ){
// b_printf("v_obj compile \n");
      RealObject *sub_src = new(std::nothrow) RealObject(progText->getLink());
      if (sub_src == NULL){
// b_printf("GProgram::compile::RealObject can't new.");
        return;
      }
      mes->def_mes(sub_src->get_fname(), VOBJ);
      compile(sub_src, SUB_SRC);
      delete sub_src;
      progText->lex();
    }
    else if (c == VAR_STRING){
      if (tc_strcmp(progText->getString(), (TC[]){L""}) == 0){ // var
        do {
          int addFlg = 0;
          if (progText->lex() != VAR_STRING){
            mes->error_mes(NULL, SYNTAX);
// b_printf("var define error NOT VAR_STRING \n");
            return;
          }
          num = 1;
          tc_strcpy(str, progText->getString());
          GVariable *p;
          if ((p = varList->getObj(str)) != NULL){
            if (p->IsKindOf() == VAR){
              mes->warning_mes(str, VAR_REDEFINE);
              addFlg++;
            }
            else{
              mes->error_mes(str, FUNC_DEFINE);
              return;
            }
          }
          if (progText->lex() == LBRCKT){
            if (progText->lex() != NUMBER){
              mes->error_mes(NULL, SYNTAX);
// b_printf("var define error NOT NUMBER \n");
              return;
            }
            num = progText->getNumber();
            if (progText->lex() != RBRCKT){
              mes->error_mes(NULL, SYNTAX);
// b_printf("var define error NOT RBRCKT \n");
              return;
            }
            progText->lex();
          }
          if (addFlg == 0){
            varList->add(str, num);
            mes->def_mes(str, VAR);
          }
        } while (progText->getReturn() == COMMA);
        if (progText->getReturn() != SEMICOL){ // ;
          mes->error_mes(NULL, SYNTAX);
// b_printf("var define error NOT SEMICOL \n");
          return;
        }
        progText->lex();
      }
      else {
        if (progText->getReturn() != VAR_STRING){ // VAR_STRING
          mes->error_mes(NULL, SYNTAX);
// b_printf("not var and not function name \n");
          return;
        }
        tc_strcpy(func_name, progText->getString());
        GVariable *p;
        if ((p = varList->getObj(func_name)) != NULL){
          if (p->IsKindOf() == VAR){
            mes->error_mes(func_name, VAR_DEFINE);
          }
          else {
            mes->error_mes(func_name, FUNC_REDEFINE);
          }
          return;
        }
        else {
          if ((tc_strcmp(func_name, (TC[]){L""}) == 0)
              && (src_type == SUB_SRC)){
            break;
          }
          varList->add(func_name, FUNC);
          mes->def_mes(func_name, FUNC);
        }
	
        if (progText->lex() != LPAR){
          mes->error_mes(NULL, SYNTAX);
// b_printf("function define error NOT LPAR \n");
          return;
        }
        while ((c = progText->lex()) != RPAR){
          if (c == EOFSYM){
            mes->error_mes(NULL, SYNTAX);
// b_printf("function define error NOT RPAR");
            return;
          }
        }
        if (progText->lex() != LBRACE){
          mes->error_mes(NULL, SYNTAX);
// b_printf("function define error NOT LBRACE \n");
          return;
        }
        FunctionStart = progList->add_dummy(); // add code
        funcVar = varList->getObj(func_name);
        funcVar->setp(FunctionStart); // set function start address
        progText->lex();
        while (progText->getReturn() != RBRACE){
          statement(); // statement
// b_printf("1 statement end \n");
        }
        progList->add_push();   // add code
        progList->add_return(); // add code
        progText->lex();
// b_printf("end of one function compile \n");
      }
    }
    else {
      mes->error_mes(NULL, SYNTAX);
      break;
//			progText->lex();
    }
  }
  // program end pointer
  progEndPointer = progList->add_dummy(); // add code
  
  mes->def_mes(NULL, (VARKIND)-1);
	
  delete pt_new;
  progText = pt_buf;
}

void GProgram::execute()
{
  GCalc *startp;
  GVariable *funcVar = varList->getObj((TC[]){L""});
	
  startup_func();
  if (funcVar == NULL){
    mes->runtime_mes((TC[]){L""}, FUNC_NODEFINE);
// b_printf("main function is noting. \n");
    return;
  }
  if (funcVar->IsKindOf() != FUNC){
    mes->runtime_mes((TC[]){L""}, FUNC_NODEFINE);
// b_printf("main is not a function. \n");
    return;
  }
  startp = funcVar->getp();
  startp->p_push(progEndPointer);
	
  progList->run(startp);
}

void GProgram::startup_func()
{
  STIME time;
  TIMEZONE tz;
  get_tim(&time, &tz);
  srand((unsigned int)time);
}

/*
extern "C"
void MAIN(M_EXECREQ *msg)
{
	KText data;
	data.setLine((TCODE *)L"򡡣Σϡ");
	data.setLine((TCODE *)L"򡡣ᡨ");
	data.setLine((TCODE *)L"ʡˡС");
	data.setLine((TCODE *)L"Σϡ");
	data.setLine((TCODE *)L"ѡ");
	data.setLine((TCODE *)L"ʡˡС");
	data.setLine((TCODE *)L"Σϡ");
	data.setLine((TCODE *)L"ѡ");
	data.setLine((TCODE *)L"ʡˡС");
	data.setLine((TCODE *)L"ݣ");
	data.setLine((TCODE *)L"Σϡݣ");
	data.setLine((TCODE *)L"ΣϡʣΣϡˡ");
	data.setLine((TCODE *)L"ΣϡʣΣϡˡ");
	data.setLine((TCODE *)L"Σϡʣˡ");
	data.setLine((TCODE *)L"Σϡʣˡ");
	data.setLine((TCODE *)L"ѡ");
	
	GProgram prog;
	prog.compile(&data);
}
*/

