#include <basic.h>
#include <tstring.h>
#include <btron/event.h>
#include <btron/vobj.h>
#include <btron/proctask.h>
#include <bstdlib.h>
#include <tad.h>
#include <new>
#include "gvar.h"
#include "subc.h"
#include "r_obj.h"

#include "krs232c.h"
#include "qvt.h"

extern OutSider grovalFunc;
extern W wid0;

GVariable::GVariable(TC *str, GVariable *p)
{
  data = new(std::nothrow) W[1];
  data[0] = 0;
  num = 1;
  name = new(std::nothrow) TC[tc_strlen(str) + 1];
  tc_strcpy(name, str);
  next = p;
}

GVariable::GVariable(TC *str, int n, GVariable *p)
{
  data = new(std::nothrow) W[n];
  for (int i = 0; i < n; i++){
    data[i] = 0;
  }
  num = n;
  name = new(std::nothrow) TC[tc_strlen(str) + 1];
  tc_strcpy(name, str);
  next = p;
}


////////////////////////////////////
//	GCalc class functions
////////////////////////////////////

W GCalc::stack[100];
int GCalc::sp = 0;
GCalc* GCalc::program_stack[50];
int GCalc::pc = 0;

///////////////////////////////////////////

void GCalcOperator::run()
{
// cout << "GCalcOperator run" << endl;
  W d1 = pop();
  W d2 = pop();
  W ans;
	
  switch(op){
  case PLUS:
    ans = d2 + d1;
    break;
  case MINUS:
    ans = d2 - d1;
    break;
  case TIMES:
    ans = d2 * d1;
    break;
  case SLASH:
    if (d1 == 0){
//      cout << "error div /0" << endl;
      return;
    }
    ans = d2 / d1;
    break;
  case MOD:
    ans = d2 % d1;
    break;
  case EQU:
    ans = ((d2 == d1) ? 1 : 0);
    break;
  case NEQ:
    ans = ((d2 != d1) ? 1 : 0);
    break;
  case LSS:
    ans = ((d2 < d1) ? 1 : 0);
    break;
  case GTR:
    ans = ((d2 > d1) ? 1 : 0);
    break;
  case LEQ:
    ans = ((d2 <= d1) ? 1 : 0);
    break;
  case GEQ:
    ans = ((d2 >= d1) ? 1 : 0);
    break;
  case OR:
    ans = (((d2 + d1) != 0) ? 1 : 0);
    break;
  case AND:
    ans = (((d2 * d1) != 0) ? 1 : 0);
    break;
  default:
    return;
  }
  push(ans);
}

///////////////////////////////////////////

void GCalcConst::run()
{
// cout << "GCalcConst run" << endl;
  push(constd);
}

///////////////////////////////////////////

void GCalcVariable::run()
{
// cout << "GCalcVariable run" << endl;
  W d;
  if (flg == 0){
    push(gp->getd());
  }
  else {
    d = pop();
    push(gp->getd(d));
  }
}

VOID GCalcUnaryMinus::run()
{
// cout << "GCalcUnaryMinus run" << endl;
  W d = pop();
  d *= -1;
  push(d);
}

///////////////////////////////////////////

GCalcOperator2::GCalcOperator2(TC *str, GVariable *p1, GCalc *p2)
  : GCalc(p2)
{
  op = new(std::nothrow) TC[tc_strlen(str) + 1];
  tc_strcpy(op, str);
  vp = p1;
  num = 0;
}

GCalcOperator2::~GCalcOperator2()
{
  delete[] op;
}

void GCalcOperator2::run()
{
// cout << "GCalcOperator2 run" << endl;
  W d1 = pop();
  W d2;
	
  if (tc_strcmp(op, (TC[]){L""}) == 0){
    vp->set(d1);
  }
  else if (tc_strcmp(op, (TC[]){L"Ρϡ"}) == 0){
    d2 = pop();
    vp->set(d1, d2);
  }
  else if (tc_strcmp(op, (TC[]){L"ܡ"}) == 0){
    d2 = vp->getd(num);
    vp->set(d2 + d1, num);
  }
  else if (tc_strcmp(op, (TC[]){L"ݡ"}) == 0){
    d2 = vp->getd(num);
    vp->set(d2 - d1, num);
  }
}

///////////////////////////////////////////

void GCalcIf::run()
{
// cout << "GCalcIf run" << endl;
  int flg = pop();
  if (flg == 0){
    this->setNext2(this->getFalseP());
  }
  else{
    this->setNext2(NULL);
  }
}

///////////////////////////////////////////

void GCalcWhile::run()
{
// cout << "GCalcWhile run" << endl;
  int flg = pop();
  if (flg == 0){
    this->setNext2(this->getFalseP());
  }
  else {
    this->setNext2(NULL);
  }
}

///////////////////////////////////////////

void GCalcFunc::run()
{
// cout << "GCalcFunc run" << endl;
  p_push(this->getNext());
}

///////////////////////////////////////////

void GCalcReturn::run()
{
// cout << "GCalcReturn run" << endl;
  this->setNext2(p_pop());
}

///////////////////////////////////////////

GCalcConstFunction::GCalcConstFunction(TC *str, GCalc *p)
  : GCalc(p)
{
  func = new(std::nothrow) TC[tc_strlen(str) + 1];
  tc_strcpy(func, str);
}

GCalcConstFunction::~GCalcConstFunction()
{
  delete[] func;
}

void GCalcConstFunction::run()
{
// cout << "GCalcConstFunction run" << endl;
  W d1 = pop();
	
  if (tc_strcmp(func, (TC[]){L""}) == 0){
    if (d1 < 0){
      d1 = -d1;
    }
// b_printf("function abs() \n");
    push(d1);
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function random() \n");
    push(rand() % d1);
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function locate() \n");
    W d2 = pop();
    grovalFunc.locate(d2, d1);
    push(0);
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function cls() \n");
    push(d1);
    grovalFunc.cls();
    push(0);
  }
  else if (tc_strcmp(func, (TC[]){L"塲"}) == 0){
// b_printf("function line_cls() \n");
    grovalFunc.line_cls(d1);
    push(0);
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function get_curX() \n");
    push(d1);
    push(grovalFunc.get_curX());
  }
  else if(tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function get_curY() \n");
    push(d1);
    push(grovalFunc.get_curY());
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function getkey() \n");
    push(d1);
    TC c;
//		grovalFunc.flushkey();
    while ((c = grovalFunc.getkey()) == TNULL){
      eventloop();
    }
    push(c);
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function getkey2() \n");
    push(d1);
//		grovalFunc.flushkey();
    eventloop();
    push(grovalFunc.getkey());
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function keyflush() \n");
    push(d1);
    grovalFunc.flushkey();
    push(0);
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
// b_printf("function beep() \n");
    W d2 = pop();
    d2 = d2 << 12;
    d2 += d1;
    sig_buz(d2);
    push(0);
  }
  else if (tc_strcmp(func, (TC[]){L"󡲣"}) == 0){
// b_printf("function rs_open() \n");
    push(krs_open(d1));
  }
  else if (tc_strcmp(func, (TC[]){L"󡲣"}) == 0){
// b_printf("function rs_close() \n");
    push(krs_close(d1));
  }
  else if (tc_strcmp(func, (TC[]){L"󡲣"}) == 0){
// b_printf("function rs_init() \n");
    // d1:stop
    W d2 = pop();	// parity
    W d3 = pop();	// length
    W d4 = pop();	// speed
    W d5 = pop();	// dev
    push(krs_init(d5, d4, d3, d2, d1));
  }
  else if (tc_strcmp(func, (TC[]){L"󡲣"}) == 0){
// b_printf("function rs_putc() \n");
    W d2 = pop();	// dev
    B c;
    c = (d1 & 0x00ff);
    push(krs_putc(d2, c));
  }
  else if (tc_strcmp(func, (TC[]){L"󡲣"}) == 0){
// b_printf("function rs_chkbuf() \n");
    push(krs_chkbuf(d1));
  }
  else if (tc_strcmp(func, (TC[]){L"󡲣"}) == 0){
// b_printf("function rs_getc() \n");
    push(krs_getc(d1));
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
    W d2 = pop();	// dev
    push(qv_getp(d2, d1));
  }
  else if (tc_strcmp(func, (TC[]){L""}) == 0){
    push(wai_prc(d1));
  }
  else{
    push(d1);
  }
}

///////////////////////////////////////////

GCalcPrintln::GCalcPrintln(TC *str, GCalc *p)
  : GCalc(p)
{
  if (str != NULL){
    string = new(std::nothrow) TC[tc_strlen(str) + 1];
    tc_strcpy(string, str);
  }
  else{
    string = NULL;
  }
}

GCalcPrintln::~GCalcPrintln()
{
  if (string != NULL){
    delete[] string;
  }
}

void GCalcPrintln::run()
{
//  cout << "GCalcPrintln run" << endl;
  if (string != NULL){
    grovalFunc.println(string);
  }
  else{
    W d = pop();
// b_printf("run println d = %d \n", d);
    grovalFunc.println(d);
  }
}

///////////////////////////////////////////

GCalcVOpen::GCalcVOpen(LINK* lnk, B* seg, GCalc *p)
  : GCalc(p)
{
// b_printf("************ GCalcVOpen constructor *************** \n");
  if (lnk != NULL){
    v_link = (LINK*)(new(std::nothrow) VLINK);
    memcpy(v_link, lnk, sizeof(VLINK));
  }
  else {
    v_link = NULL;
  }
  UH *buf = (UH*)seg;
  int n = buf[1] + sizeof(TADSEG);
  v_seg = new(std::nothrow) B[n];
  memcpy(v_seg, seg, n);
}

GCalcVOpen::~GCalcVOpen()
{
  if (v_link != NULL){
    delete v_link;
  }
  if (v_seg != NULL){
    delete[] v_seg;
  }
}

void GCalcVOpen::run()
{
  VLINK *vp = (VLINK*)v_link;
  VFPTR com_seg;
	
  com_seg.vobj = (VOBJSEG*)&v_seg[4];
  W v_id = oreg_vob(vp, com_seg.vobj, wid0, V_NODISP);
  if (v_id >= 0){
    oexe_apg(v_id, 0);
    // osta_prc(v_id, wid0);
    VLINK vl;
    do {
      oget_vob(v_id, &vl, com_seg.vobj, sizeof(VOBJSEG), NULL);
      vl.attr &= 0x2000;
      eventloop();
    } while (vl.attr == 0);
    do {
      oget_vob(v_id, &vl, com_seg.vobj, sizeof(VOBJSEG), NULL);
      vl.attr &= 0x2000;
      eventloop();
    } while (vl.attr != 0);
    //odel_vob(v_id, 0);
  }
}


///////////////////////////////////////////
//	GProgramList class function
///////////////////////////////////////////

GVariableList::GVariableList()
{
  if ((pEnd = new(std::nothrow) GVariable((TC[]){L""}, NULL)) == NULL){
// b_printf("GVariableList::constructor can't new \n");
  }
  pStart = pEnd;
  pCursor = pStart;
}

int GVariableList::add(TC *str, int n)
{
  GVariable *p;
  if ((p = new(std::nothrow) GVariable(str, n, NULL)) == NULL){
// b_printf("GVariableList::add can't new \n");
  }
  pEnd->setNext(p);
  pEnd = p;
  return 0;
}

int GVariableList::add(TC *str, VARKIND k)
{
  GFunction *p;
	
  if (k == VAR){
    add(str);
  }
  else {
    if ((p = new(std::nothrow) GFunction(str, NULL)) == NULL){
// b_printf("GVariableList::add can't new \n");
    }
    pEnd->setNext(p);
    pEnd = p;
  }
  return 0;
}

GVariable *GVariableList::getObj(TC *str)
{
  this->seekTop();
  while (pCursor != NULL){
    if (tc_strcmp(str, pCursor->getName()) == 0){
      return pCursor;
    }
    pCursor = pCursor->getNext();
  }
  return NULL;
}

void GVariableList::deleteAll()
{
  GVariable *p;
	
  while (pStart != NULL){
    p = pStart;
    pStart = pStart->getNext();
    delete p;
  }
  pEnd = new(std::nothrow) GVariable((TC[]){L""}, NULL);
  pStart = pEnd;
  pCursor = pStart;
}


GProgramList::GProgramList()
{
  GCalcConst *p = new(std::nothrow) GCalcConst(0, NULL);
  dummy = new(std::nothrow) GVariable((TC[]){L""}, NULL);
	
  if (dummy == NULL){
// b_printf("GProgramList COnstructor(dummy) can't new \n");
  }
  if (p == NULL){
// b_printf("GProgramList COnstructor(p) can't new \n");
  }
	
  pEnd = p;
  pStart = pEnd;
  pCursor = pStart;
}

GProgramList::~GProgramList()
{
  deleteAll();
  delete pEnd;
  delete dummy;
}

int GProgramList::add_op(SYMBOL c)
{
  GCalcOperator *vp = new(std::nothrow) GCalcOperator(c, NULL);
  if (vp == NULL){
// b_printf("GProgramList::add_op can't open \n");
  }
  return add(vp);
}

int GProgramList::add_const(W d)
{
  GCalcConst *vp = new(std::nothrow) GCalcConst(d, NULL);
  if (vp == NULL){
// b_printf("GProgramList::add_const can't open \n");
  }
  return add(vp);
}

int GProgramList::add_var(int flg, GVariable *p)
{
  GCalcVariable *vp = new(std::nothrow) GCalcVariable(p, flg, NULL);
  if (vp == NULL){
// b_printf("GProgramList::add_var can't open \n");
  }
  return add(vp);
}

int GProgramList::add_op2(TC* op, GVariable *p)
{
  GCalcOperator2 *vp = new(std::nothrow) GCalcOperator2(op, p, NULL);
  if (vp == NULL){
// b_printf("GProgramList::add_op2 can't open \n");
  }
  return add(vp);
}

int GProgramList::add_um()
{
  GCalcUnaryMinus *p = new(std::nothrow) GCalcUnaryMinus(NULL);
  if (p == NULL){
// b_printf("GProgramList::add_um can't open \n");
  }
  return add(p);
}

GCalcIf *GProgramList::add_if()
{
  GCalcIf *p = new(std::nothrow) GCalcIf(NULL);
  if (p == NULL){
// b_printf("GProgramList::add_if can't open \n");
  }
  add(p);
  return p;
}

GCalcWhile *GProgramList::add_while()
{
  GCalcWhile *p = new(std::nothrow) GCalcWhile(NULL);
  if (p == NULL){
// b_printf("GProgramList::add_while can't open \n");
  }
  add(p);
  return p;
}

GCalcDummy *GProgramList::add_dummy()
{
  GCalcDummy *p = new(std::nothrow) GCalcDummy(NULL);
  if (p == NULL){
// b_printf("GProgramList::add_dummy can't open \n");
  }
  add(p);
  return p;
}

int GProgramList::add_push()
{
  GCalcPush *p = new(std::nothrow) GCalcPush(NULL);
  if (p == NULL){
// b_printf("GProgramList::add_push can't open \n");
  }
  return add(p);
}

int GProgramList::add_pop()
{
  GCalcPop *p = new(std::nothrow) GCalcPop(NULL);
  if (p == NULL){
// b_printf("GProgramList::add_pop can't open \n");
  }
  return add(p);
}

GCalcFunc *GProgramList::add_func()
{
  GCalcFunc *p = new(std::nothrow) GCalcFunc(NULL);
  if (p == NULL){
// b_printf("GProgramList::add_func can't open \n");
  }
  add(p);
// b_printf("new function add to program list \n");
  return p;
}

GCalcReturn *GProgramList::add_return()
{
  GCalcReturn *p = new(std::nothrow) GCalcReturn(NULL);
  if (p == NULL){
// b_printf("GProgramList::add_return can't open \n");
  }
// b_printf("GProgramList::add_return open \n");
  add(p);
  return p;
}

int GProgramList::add_constFunc(TC *str)
{
  GCalcConstFunction *p = new(std::nothrow) GCalcConstFunction(str, NULL);
  if (p == NULL){
// b_printf("GProgramList::add_constFunc can't open \n");
  }
  return add(p);
}

int GProgramList::add_println(TC *str)
{
  GCalcPrintln *p = new(std::nothrow) GCalcPrintln(str, NULL);
  if (p == NULL){
// b_printf("GProgramList::add_println can't open \n");
  }
  return add(p);
}

int GProgramList::add_vopen(LINK* lnk, B *seg)
{
  GCalcVOpen *p = new(std::nothrow) GCalcVOpen(lnk, seg, NULL);
  if (p == NULL){
// b_printf("GProgramList::add_vopen can't open \n");
  }
  return add(p);
}

int GProgramList::add(GCalc *p)
{
  pEnd->setNext(p);
  pEnd = p;
  return 0;
}

void GProgramList::deleteAll()
{
  GCalc *p;
  while (pStart != NULL){
    p = pStart;
    pStart = pStart->getNext();
    delete p;
  }
  pEnd = new(std::nothrow) GCalcConst(0, NULL);
  pStart = pEnd;
  pCursor = pStart;
}

void GProgramList::run(GCalc *sp)
{
  this->seekTop();
  if(sp != NULL){
    pCursor = sp;
  }
  while (pCursor != NULL){
    pCursor->run();
    if (pCursor->getNext2() == NULL){
      pCursor = pCursor->getNext();
    }
    else {
      pCursor = pCursor->getNext2();
    }
  }
}






/*
extern "C"
void MAIN(M_EXECREQ *msg)
{
	GVariableList list;
	
	list.add((TCODE *)L"");
	list.add((TCODE *)L"");
	
	GVariable *p1, *p2;
	if((p1 = list.getObj((TCODE *)L"")) == NULL){
//		cout << "error p1" << endl;
		return ;
	}
	if((p2 = list.getObj((TCODE *)L"")) == NULL){
//		cout << "error p2" << endl;
		return ;
	}
	
	p1->set(2);
	p2->set(34);
	
	b_printf("kaneko = %d \n", p1->getd());
	b_printf("yutaka = %d \n", p2->getd());
//	cout << "kaneko = " << p1->getd() << endl;
//	cout << "yutaka = " << p2->getd() << endl;
	
	GProgramList calc;
	
	calc.add_var(p1);
	calc.add_var(p2);
	calc.add_op(MINUS);
	calc.add_um();
	calc.add_op2((TCODE*)L"", p1);

	
//	calc.run();
//	cout << "ans = " << p1->getd() << endl;
	
	calc.deleteAll();
	calc.add_var(p1);
	calc.add_var(p2);
	calc.add_op(SLASH);
	calc.add_op2((TCODE *)L"", p1);
	p1->set(333);
	p2->set(444);
	b_printf("kaneko = %d \n", p1->getd());
	b_printf("yutaka = %d \n", p2->getd());
//	calc.run();
//	cout << "ans = " << p1->getd() << endl;
}
*/

