#ifndef	_GVAR_H
#define _GVAR_H

#include "gcommon.h"
#include "r_obj.h"

class GCalc;

////////////////////////////////////
//	GVariable
////////////////////////////////////

class GVariable
{
private:
  W  *data;
  TC *name;
  GVariable *next;
protected:
  int num;
	
public:
  GVariable(TC *str, GVariable *p);
  GVariable(TC *str, int n, GVariable *p);
  virtual ~GVariable(){
    delete[] data;
    delete[] name;
  }
	
  W set(W d) {
    return set(d, 0);
  }
  W set(W d, W n) {
    return (data[n] = d);
  }
  W getd() {
    return getd(0);
  }
  W getd(W n) {
    return data[n];
  }
  TC* getName() {
    return name;
  }
	
  void setNext(GVariable *p) {
    next = p;
  }
  GVariable* getNext() {
    return next;
  }
	
  virtual void setp(GCalc *p) {}
  virtual GCalc* getp() {
    return NULL;
  }
	
  VARKIND IsKindOf() {
    if (num == 0){
      return (VARKIND)FUNC;
    }
    else {
      return (VARKIND)VAR;
    }
  }

};

class GFunction
  : public GVariable
{
  GCalc *start_point;
	
public:
  GFunction(TC *str, GVariable *p) : GVariable(str, p) {
    start_point = NULL;
    num = 0;
  }
  virtual ~GFunction() {}
	
	virtual VOID setp(GCalc *p) { start_point = p; }
	virtual GCalc *getp(VOID) { return start_point; }
};


////////////////////////////////////
//	GVariableList
////////////////////////////////////

class GVariableList
{
  GVariable *pStart;
  GVariable *pEnd;
  GVariable *pCursor;
	
public:
  GVariableList(VOID);

  ~GVariableList() {
    deleteAll();
    delete pEnd;
  }
	
  int add(TC *str) {
    return add(str, 1);
  }
  int add(TC *str, int n);
  int add(TC *str, VARKIND k);
  void seekTop() {
    pCursor = pStart;
  }

  GVariable* getObj(TC* str);
  void deleteAll();
};


////////////////////////////////////
//	GCalc
////////////////////////////////////
class GCalc
{
protected:
  GCalc *next;
  GCalc *next2;
  static W stack[];
  static int sp;
  static GCalc* program_stack[];
  static int pc;
	
public:
  GCalc(GCalc *p){
    next = p;
    next2 = NULL;
  }
  virtual ~GCalc() {}
	
  virtual void run() = 0;
	
  void setNext(GCalc *p) {
    next = p;
  }
  GCalc* getNext() {
    return next;
  }
  void setNext2(GCalc *p) {
    next2 = p;
  }
  GCalc* getNext2() {
    return next2;
  }
	
  void push(W d) {
    stack[sp++] = d;
  }
  W pop() {
    if (sp < 0){
      return NULL;
    }
    return stack[--sp];
  }
  void p_push(GCalc *p) {
    program_stack[pc++] = p;
  }
  GCalc* p_pop() {
    if (pc < 0){
      return NULL;
    }
    return program_stack[--pc];
  }
};


class GCalcOperator
  : public GCalc
{
  SYMBOL op;
	
public:
  GCalcOperator(SYMBOL c, GCalc *p) : GCalc(p) { op = c; }
  virtual ~GCalcOperator() {}
	
  virtual void run();
};


class GCalcConst
  : public GCalc
{
  W constd;
	
public:
  GCalcConst(W d, GCalc *p) : GCalc(p) {
    constd = d;
  }
  virtual ~GCalcConst() {}
	
  virtual void run();
};


class GCalcOperator2
  : public GCalc
{
  TC *op;
  GVariable *vp;
  int num;
	
public:
  GCalcOperator2(TC *str, GVariable *p1, GCalc *p2);
  virtual ~GCalcOperator2();
	
  virtual void run();
};


class GCalcUnaryMinus
  : public GCalc
{
public:
  GCalcUnaryMinus(GCalc *p) : GCalc(p) {}
  virtual ~GCalcUnaryMinus() {}

  virtual void run();
};


class GCalcVariable
  : public GCalc
{
  GVariable *gp;
  int flg; // 0 : normal, 1 : array
	
public:
  GCalcVariable(GVariable *vp, int f, GCalc *p) : GCalc(p) {
    gp = vp;
    flg = f;
  }
  virtual ~GCalcVariable() {}
	
  virtual void run();
};

class GCalcIf
  : public GCalc
{
  GCalc *false_p;
	
public:
  GCalcIf(GCalc *p) : GCalc(p) {
    false_p = NULL;
  }
  virtual ~GCalcIf() {}
  void setFalseP(GCalc *p) {
    false_p = p;
  }
  GCalc *getFalseP() {
    return false_p;
  }
	
  virtual void run();
};

class GCalcWhile
  : public GCalc
{
  GCalc *false_p;
	
public:
  GCalcWhile(GCalc *p) : GCalc(p) {
    false_p = NULL;
  }
  virtual ~GCalcWhile() {}
  VOID setFalseP(GCalc *p) {
    false_p = p;
  }
  GCalc* getFalseP() {
    return false_p;
  }

  virtual void run();
};

class GCalcDummy
  : public GCalc
{
public:
  GCalcDummy(GCalc *p) : GCalc(p) {}
  virtual ~GCalcDummy() {}
	
  virtual void run() {}
};

class GCalcPush
  : public GCalc
{
public:
  GCalcPush(GCalc *p) : GCalc(p) {}
  virtual ~GCalcPush() {}
	
  virtual void run() {
    push(0);
  }
};

class GCalcPop
  : public GCalc
{
public:
  GCalcPop(GCalc *p) : GCalc(p) {}
  virtual ~GCalcPop() {}
	
  virtual void run() {
    pop();
  }
};

class GCalcFunc
  : public GCalc
{
public:
  GCalcFunc(GCalc *p) : GCalc(p) {}
  virtual ~GCalcFunc() {}
	
  virtual VOID run();
};

class GCalcReturn
  : public GCalc
{
public:
  GCalcReturn(GCalc *p) : GCalc(p) {}
  virtual ~GCalcReturn() {}
	
  virtual void run();
};

class GCalcConstFunction
  : public GCalc
{
  TC* func;
	
public:
  GCalcConstFunction(TC *str, GCalc *p);
  virtual ~GCalcConstFunction();
	
  virtual void run();
};

class GCalcPrintln
  : public GCalc
{
  TC *string;
	
public:
  GCalcPrintln(TC *str, GCalc *p);
  virtual ~GCalcPrintln();
	
  virtual void run();
};

class GCalcVOpen
  : public GCalc
{
  LINK* v_link;
  B* v_seg;

public:
  GCalcVOpen(LINK* lnk, B* seg, GCalc *p);
  virtual ~GCalcVOpen();
	
  virtual void run();
};


////////////////////////////////////
//	GProgramList
////////////////////////////////////

class GProgramList
{
  GCalc *pStart;
  GCalc *pEnd;
  GCalc *pCursor;
  GVariable *dummy;
	
public:
  GProgramList();
  ~GProgramList();
	
  int add(GCalc *p);
  int add_op(SYMBOL c);
  int add_const(W d);
  int add_var(GVariable *p) {
    return add_var(0, p);
  }
  // flg = 0: normal var, 1: array var
  int add_var(int flg, GVariable *p);
  int add_op2(TC *op, GVariable *p);
  int add_um();
  int add_push();
  int add_pop();
  int add_constFunc(TC *str);
  int add_println(TC *str);
  int add_vopen(LINK* lnk, B *seg);
  GCalcIf *add_if();
  GCalcWhile *add_while();
  GCalcDummy *add_dummy();
  GCalcFunc *add_func();
  GCalcReturn *add_return();
	
  W pop(VOID) {
    return pStart->pop();
  }
  void seekTop() {
    pCursor = pStart;
  }
  void deleteAll();
  void run(GCalc *sp);
	
  GCalc *getEndP() {
    return pEnd;
  }
};

#endif	// end of gvar.h

