#include <basic.h>
#include <tcode.h>
#include <btron/dp.h>
#include <btron/hmi.h>
#include <tstring.h>
#include <bstdio.h>
#include <vector>
#include "textscr.h"

void redisp(void);

TextScreen::TextScreen(W id)
{
  wid = id;
  gid = wget_gid(wid);
  maxX = CHAR_NUM_X - 1;
  maxY = CHAR_NUM_Y - 1;
  cursorX = 0;
  cursorY = 0;
  charSizeX = CHAR_SIZE_H;
  charSizeY = CHAR_SIZE_V;
  static FSSPEC fs = {{TNULL}, FTC_DEFAULT, 0, {8, 16}};
  gset_fon(gid, &fs);
  COLOR fg = 0x1000ff00;
  COLOR bg = 0x10000000;
  gset_chc(gid, fg, bg);
  cls();
}

int TextScreen::set_txpos(int x)
{
  int flg = chk_xpos(x);
  if (flg < 0){
    x = 0;
  }
  else if (flg > 0){
    x = maxX;
  }
  return x;
}

int TextScreen::set_typos(int y)
{
  int flg = chk_ypos(y);
  if (flg < 0){
    y = 0;
  }
  else if (flg > 0){
    y = maxY;
  }
  return y;
}

void TextScreen::redisplay(int x1, int y1, int x2, int y2)
{
  int tx1 = set_txpos(get_txpos(x1) - 1);
  int tx2 = set_txpos(get_txpos(x2) + 1);
  int ty1 = set_typos(get_typos(y1) - 1);
  int ty2 = set_typos(get_typos(y2) + 1);
  disp_rect(tx1, ty1, tx2, ty2);
}

void TextScreen::text_scrol()
{
  for (int y = 0; y < maxY; y++){
    for (int x = 0; x <= maxX; x++){
      screenBuff[get_add(x, y)] = screenBuff[get_add(x, y+1)];
    }
  }
  for (int x = 0; x <= maxX; x++){
    screenBuff[get_add(x, maxY)] = TK_KSP;
  }
  redisplay();
}

void TextScreen::cls()
{
  for (int y = 0; y <= maxY; y++){
    for (int x = 0; x <= maxX; x++){
      screenBuff[get_add(x, y)] = TK_KSP;
    }
  }
  cursorX = 0;
  cursorY = 0;
  redisplay();
}

void TextScreen::line_cls(int n)
{
  if (chk_ypos(n) != 0){
    return;
  }
  for (int x = 0; x <= maxX; x++){
    screenBuff[get_add(x, n)] = TK_KSP;
  }
  disp_line(0, n, maxX+1);
}

int TextScreen::chk_size(int x, int min, int max)
{
  if (x < min){
    return -1;
  }
  if (x > max){
    return 1;
  }
  return 0;
}

void TextScreen::disp_line(int x, int y, int len)
{
  std::vector<TC> str;
	
  for (int i = 0; i < len; i++){
    str.push_back(screenBuff[get_add(x+i, y)]);
  }
  str.push_back(TNULL);
	
  int gx = get_gxpos(x);
  int gy = get_gypos(y);
  gdra_stp(gid, gx, gy, &(str[0]), len, G_STORE);
}

void TextScreen::disp_rect(int x1, int y1, int x2, int y2)
{
  for (int i = y1; i <= y2; i++){
    disp_line(x1, i, x2 - x1 + 1);
  }
}

int TextScreen::add_xpos()
{
  cursorX++;
  if (chk_xpos(cursorX) != 0){
    cursorX = 0;
    return add_ypos();
  }
  return 0;
}

int TextScreen::add_ypos()
{
  cursorY++;
  if (chk_ypos(cursorY) != 0){
    cursorY--;
    text_scrol();
    return -1;
  }
  return 0;
}

int TextScreen::copy_str(TC *str, int n)
{
  TC *p = str;
  int cnt = 0;
  int startY = cursorY;
	
  while (n != 0){
    if (*p == TNULL){
      break;
    }
    if(*p != TK_BSLH){
      screenBuff[get_add(cursorX, cursorY)] = *p++;
      if (add_xpos() < 0){
        //        break;
      }
    }
    else {
      p++;
      if (*p++ == TK_n){
        cursorX = 0;
        if (add_ypos() < 0){
          //          break;
        }
      }
    }
    n--;
    cnt++;
  }
  disp_rect(0, startY, maxX, cursorY);
  return cnt;
}


void TextScreen::locate(int x, int y)
{
  if ((chk_xpos(x) != 0) || (chk_ypos(y) != 0)){
    return;
  }
  cursorX = x;
  cursorY = y;
}

void TextScreen::println(TC *str)
{
  copy_str(str, tc_strlen(str));
}

void TextScreen::println(int d)
{
  char buf[200];
  TC str[100];

  // wb_sprintfˡ
  sprintf(buf, "%d", d);
  eucstotcs(str, (B*)buf);

  println(str);
}
